@@ -2,13 +2,16 @@
    |                     Mobile Robot Programming Toolkit (MRPT)            |
    |                          https://www.mrpt.org/                         |
    |                                                                        |
-   | Copyright (c) 2005-2020, Individual contributors, see AUTHORS file     |
+   | Copyright (c) 2005-2023, Individual contributors, see AUTHORS file     |
    | See: https://www.mrpt.org/Authors - All rights reserved.               |
    | Released under BSD License. See: https://www.mrpt.org/License          |
    +------------------------------------------------------------------------+ */
 
 #include "_DSceneViewerMain.h"
+
+#include <mrpt/system/string_utils.h>  // firstNLines()
 #include <wx/app.h>
+
 #include "CDlgCamTracking.h"
 #include "CDlgPLYOptions.h"
 
@@ -22,8 +25,6 @@
 #include <wx/tglbtn.h>
 //*)
 
-#include "CDialogOptions.h"
-
 #include <wx/busyinfo.h>
 #include <wx/choicdlg.h>
 #include <wx/colordlg.h>
@@ -37,14 +38,29 @@
 #include <wx/progdlg.h>
 #include <wx/textdlg.h>
 
+#include "CDialogOptions.h"
+
 #if defined(__WXMSW__)
 const std::string iniFileSect("CONF_WIN");
 #elif defined(__UNIX__)
 const std::string iniFileSect("CONF_LIN");
 #endif
 
+#include "../wx-common/Applications.xpm"
+#include "../wx-common/ArrowLeft2.xpm"
+#include "../wx-common/Folderdownloads.xpm"
+#include "../wx-common/Qmark.xpm"
+#include "../wx-common/empty_file.xpm"
+#include "../wx-common/icon_backward.xpm"
+#include "../wx-common/icon_cubes.xpm"
+#include "../wx-common/icon_forward.xpm"
+#include "../wx-common/icon_play.xpm"
+#include "../wx-common/icon_record.xpm"
+#include "../wx-common/icon_undo.xpm"
 #include "../wx-common/mrpt_logo.xpm"
 #include "imgs/icono_main.xpm"
+//
+#include "../wx-common/return_bitmap.h"
 
 #if !wxUSE_GLCANVAS
 #error "OpenGL required: set wxUSE_GLCANVAS to 1 and rebuild wxWidgets"
@@ -55,28 +71,27 @@ const std::string iniFileSect("CONF_LIN");
 #include <mrpt/gui/about_box.h>
 #include <mrpt/io/CFileGZInputStream.h>
 #include <mrpt/io/CFileGZOutputStream.h>
+#include <mrpt/maps/CColouredPointsMap.h>
+#include <mrpt/maps/CPointsMap.h>
 #include <mrpt/opengl/CAngularObservationMesh.h>  // It's in lib mrpt-maps
 #include <mrpt/opengl/CAssimpModel.h>
 #include <mrpt/opengl/CFBORender.h>
 #include <mrpt/opengl/CGridPlaneXY.h>
-#include <mrpt/opengl/COpenGLScene.h>
 #include <mrpt/opengl/CPlanarLaserScan.h>  // It's in lib mrpt-maps
 #include <mrpt/opengl/CPointCloud.h>
 #include <mrpt/opengl/CPointCloudColoured.h>
+#include <mrpt/opengl/Scene.h>
 #include <mrpt/opengl/stock_objects.h>
 #include <mrpt/serialization/CArchive.h>
 #include <mrpt/system/CDirectoryExplorer.h>
 #include <mrpt/system/CTicTac.h>
 #include <mrpt/system/filesystem.h>
 #include <mrpt/system/string_utils.h>
-
-#include <mrpt/maps/CColouredPointsMap.h>
-#include <mrpt/maps/CPointsMap.h>
 #if MRPT_HAS_LIBLAS
 #include <mrpt/maps/CPointsMap_liblas.h>
 #endif
 
-const mrpt::maps::CColouredPointsMap dummy_map;  // this is to enforce to load
+const mrpt::maps::CColouredPointsMap dummy_map;	 // this is to enforce to load
 // the mrpt-maps DLL, then
 // register all OpenGL classes
 // defined there.
@@ -97,6 +112,18 @@ wxBitmap MyArtProvider::CreateBitmap(
 	if (id == wxART_MAKE_ART_ID(MAIN_ICON)) return wxBitmap(icono_main_xpm);
 	if (id == wxART_MAKE_ART_ID(IMG_MRPT_LOGO)) return wxBitmap(mrpt_logo_xpm);
 
+	RETURN_BITMAP(wxART_NORMAL_FILE, empty_file_xpm)
+	RETURN_BITMAP(wxART_FILE_OPEN, Folderdownloads_xpm);
+	RETURN_BITMAP(wxART_REMOVABLE, icon_play_xpm);
+	RETURN_BITMAP(wxART_HELP_BOOK, Qmark_xpm);
+	RETURN_BITMAP(wxART_QUIT, ArrowLeft2_xpm);
+	RETURN_BITMAP(wxART_GO_FORWARD, icon_forward_xpm);
+	RETURN_BITMAP(wxART_GO_BACK, icon_backward_xpm);
+	RETURN_BITMAP(wxART_FIND, Applications_xpm);
+	RETURN_BITMAP(wxART_TICK_MARK, icon_cubes_xpm);
+	RETURN_BITMAP(wxART_REDO, icon_undo_xpm);
+	RETURN_BITMAP(wxART_HARDDISK, icon_record_xpm);
+
 	// Any wxWidgets icons not implemented here
 	// will be provided by the default art provider.
 	return wxNullBitmap;
@@ -105,19 +132,6 @@ wxBitmap MyArtProvider::CreateBitmap(
 // Used for feedback from the glcanvas component to its parent.
 _DSceneViewerFrame* theWindow = nullptr;
 
-#ifdef MRPT_OS_WINDOWS
-// Windows:
-#include <windows.h>
-#endif
-
-#ifdef MRPT_OS_APPLE
-#include <OpenGL/gl.h>
-#include <OpenGL/glu.h>
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif
-
 #include <mutex>
 
 using namespace mrpt;
@@ -155,22 +169,32 @@ wxLogWindow* logWin = nullptr;
 
 void saveLastUsedDirectoryToCfgFile(const std::string& fil)
 {
-	try
-	{
-		iniFile->write(iniFileSect, "LastDir", extractFileDirectory(fil));
-	}
-	catch (const std::exception& e)
-	{
-		wxMessageBox(mrpt::exception_to_str(e), _("Exception"), wxOK);
-	}
+	WX_START_TRY
+
+	iniFile->write(iniFileSect, "LastDir", extractFileDirectory(fil));
+
+	WX_END_TRY
 }
 
 void CMyGLCanvas::OnRenderError(const wxString& str)
 {
-	if (!logWin) logWin = new wxLogWindow(this, wxT("Log window"), false);
+	const size_t maxLines = 7;
+	const std::string sErr =
+		mrpt::system::firstNLines(str.ToStdString(), maxLines);
 
-	wxLogError(str);
-	logWin->Show();
+	if (!logWin)
+	{
+		logWin = new wxLogWindow(this, wxT("Log window"), false);
+		logWin->Show();
+	}
+
+	static double lastErrMsg = 0;
+	double tNow = mrpt::Clock::nowDouble();
+	if (tNow - lastErrMsg > 10.0)
+	{
+		lastErrMsg = tNow;
+		wxLogError(wxString(sErr));
+	}
 }
 
 void CMyGLCanvas::OnPreRender() {}
@@ -205,8 +229,7 @@ void CMyGLCanvas::OnPostRenderSwapBuffers(double At, wxPaintDC& dc)
 	double estimatedFPS;
 	static double meanEstimatedFPS = 1;
 
-	if (At > 0)
-		estimatedFPS = 1 / At;
+	if (At > 0) estimatedFPS = 1 / At;
 	else
 		estimatedFPS = 0;
 
@@ -236,9 +259,6 @@ void CMyGLCanvas::OnCharCustom(wxKeyEvent& event)
 	{
 		try
 		{
-			CTicTac tictac;
-			tictac.Tic();
-
 			// Load a different file:
 
 			// First, build a list of files in the directory:
@@ -250,7 +270,8 @@ void CMyGLCanvas::OnCharCustom(wxKeyEvent& event)
 				timeDifference(lastUpdateOfList, curTime) > 2)
 			{
 				CDirectoryExplorer::explore(
-					extractFileDirectory(loadedFileName),  // path
+					extractFileDirectory(
+						mrpt::system::toAbsolutePath(loadedFileName)),	// path
 					FILE_ATTRIB_ARCHIVE,  // mask
 					lstFiles);
 
@@ -261,10 +282,8 @@ void CMyGLCanvas::OnCharCustom(wxKeyEvent& event)
 				lastUpdateOfList = curTime;
 			}
 
-			// cout << "Time to build file list: " << tictac.Tac() << endl;
-
 			string curFileName = extractFileName(loadedFileName) + string(".") +
-								 extractFileExtension(loadedFileName);
+				extractFileExtension(loadedFileName);
 
 			// Find the current file:
 			CDirectoryExplorer::TFileInfoList::iterator it;
@@ -274,42 +293,28 @@ void CMyGLCanvas::OnCharCustom(wxKeyEvent& event)
 				if (!os::_strcmpi(it->name.c_str(), curFileName.c_str()))
 				{
 					// Look for the desired file:
+					std::optional<std::string> fil;
 					if (evkey == WXK_LEFT)
 					{
-						if (i > 0)
-							theWindow->loadFromFile(
-								lstFiles[i - 1].wholePath, true);
-						return;
+						if (i > 0) fil = lstFiles[i - 1].wholePath;
 					}
 					else
 					{
 						if (i < (lstFiles.size() - 1))
-							theWindow->loadFromFile(
-								lstFiles[i + 1].wholePath, true);
-						else
-						{
-							// This was the last file
-						}
-						return;
+							fil = lstFiles[i + 1].wholePath;
 					}
+
+					if (fil.has_value())
+						theWindow->loadFromFile(fil.value(), true);
+					return;
 				}
 			}
 		}
-		catch (std::exception&)
+		catch (const std::exception& e)
 		{
-			// cerr << "*EXCEPTION* while determining next/previous file:\n" <<
-			// mrpt::exception_to_str(e) << endl;
+			std::cerr << "*EXCEPTION* while determining next/previous file:\n"
+					  << e.what() << std::endl;
 		}
-		catch (...)
-		{
-		}
-	}
-
-	if (evkey == WXK_UP || evkey == WXK_DOWN)
-	{
-		// Move the camera forward-backward:
-		// ...
-		// Refresh(false);
 	}
 }
 
@@ -344,6 +349,7 @@ const long _DSceneViewerFrame::ID_MENUITEM30 = wxNewId();
 const long _DSceneViewerFrame::ID_MENUITEM12 = wxNewId();
 const long _DSceneViewerFrame::ID_MENUITEM23 = wxNewId();
 const long _DSceneViewerFrame::ID_MENUITEM18 = wxNewId();
+const long _DSceneViewerFrame::ID_MENUITEM_PRINT_TEXT = wxNewId();
 const long _DSceneViewerFrame::idMenuQuit = wxNewId();
 const long _DSceneViewerFrame::ID_MENUITEM24 = wxNewId();
 const long _DSceneViewerFrame::ID_MENUITEM26 = wxNewId();
@@ -397,7 +403,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	wxMenu* Menu2;
 
 	Create(
-		parent, id, _("3DSceneViewer - Part of the MRPT project"),
+		parent, id, _("SceneViewer3D - Part of the MRPT project"),
 		wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE, _T("id"));
 	SetMinSize(wxSize(150, 100));
 	{
@@ -419,8 +425,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnNew->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_NORMAL_FILE")), wxART_TOOLBAR));
 	btnNew->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnNew, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnNew, 1, wxEXPAND, 1);
 	btnToolbarOpen = new wxCustomButton(
 		this, ID_BUTTON2, _("  Open... "),
 		wxArtProvider::GetBitmap(
@@ -430,8 +435,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnToolbarOpen->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FILE_OPEN")), wxART_TOOLBAR));
 	btnToolbarOpen->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnToolbarOpen, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnToolbarOpen, 1, wxEXPAND, 1);
 	StaticLine1 = new wxStaticLine(
 		this, ID_STATICLINE1, wxDefaultPosition, wxSize(50, -1),
 		wxLI_HORIZONTAL, _T("ID_STATICLINE1"));
@@ -447,8 +451,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnPrev->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_BACK")), wxART_TOOLBAR));
 	btnPrev->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnPrev, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnPrev, 1, wxEXPAND, 1);
 	btnNext = new wxCustomButton(
 		this, ID_BUTTON4, _("  Next  "),
 		wxArtProvider::GetBitmap(
@@ -458,8 +461,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnNext->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_GO_FORWARD")), wxART_TOOLBAR));
 	btnNext->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnNext, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnNext, 1, wxEXPAND, 1);
 	btnReload = new wxCustomButton(
 		this, ID_BUTTON5, _("  Reload  "),
 		wxArtProvider::GetBitmap(
@@ -470,8 +472,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_REDO")), wxART_TOOLBAR));
 	btnReload->SetMargins(wxSize(5, 5));
 	FlexGridSizer2->Add(
-		btnReload, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP,
-		wxDLG_UNIT(this, wxSize(1, 0)).GetWidth());
+		btnReload, 1, wxEXPAND, wxDLG_UNIT(this, wxSize(1, 0)).GetWidth());
 	StaticLine2 = new wxStaticLine(
 		this, ID_STATICLINE2, wxDefaultPosition, wxSize(50, -1),
 		wxLI_HORIZONTAL, _T("ID_STATICLINE2"));
@@ -487,8 +488,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnOptions->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_FIND")), wxART_TOOLBAR));
 	btnOptions->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnOptions, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnOptions, 1, wxEXPAND, 1);
 	btnOrtho = new wxCustomButton(
 		this, ID_BUTTON7, _("  Ortho  "),
 		wxArtProvider::GetBitmap(
@@ -498,8 +498,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnOrtho->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_TICK_MARK")), wxART_TOOLBAR));
 	btnOrtho->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnOrtho, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnOrtho, 1, wxEXPAND, 1);
 	btnAutoplay = new wxCustomButton(
 		this, ID_BUTTON8, _("  Autoplay  "),
 		wxArtProvider::GetBitmap(
@@ -509,8 +508,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnAutoplay->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_REMOVABLE")), wxART_TOOLBAR));
 	btnAutoplay->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnAutoplay, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnAutoplay, 1, wxEXPAND, 1);
 	btnCapture = new wxCustomButton(
 		this, ID_BUTTON9, _("  Capture  "),
 		wxArtProvider::GetBitmap(
@@ -520,8 +518,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnCapture->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_HARDDISK")), wxART_TOOLBAR));
 	btnCapture->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnCapture, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnCapture, 1, wxEXPAND, 1);
 	StaticLine3 = new wxStaticLine(
 		this, ID_STATICLINE3, wxDefaultPosition, wxSize(50, -1),
 		wxLI_HORIZONTAL, _T("ID_STATICLINE3"));
@@ -537,8 +534,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnAbout->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_HELP_BOOK")), wxART_TOOLBAR));
 	btnAbout->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnAbout, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
+	FlexGridSizer2->Add(btnAbout, 1, wxEXPAND, 1);
 	btnQuit = new wxCustomButton(
 		this, ID_BUTTON11, _("  Quit  "),
 		wxArtProvider::GetBitmap(
@@ -548,10 +544,8 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	btnQuit->SetBitmapDisabled(wxArtProvider::GetBitmap(
 		wxART_MAKE_ART_ID_FROM_STR(_T("wxART_QUIT")), wxART_TOOLBAR));
 	btnQuit->SetMargins(wxSize(5, 5));
-	FlexGridSizer2->Add(
-		btnQuit, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 1);
-	FlexGridSizer1->Add(
-		FlexGridSizer2, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 0);
+	FlexGridSizer2->Add(btnQuit, 1, wxEXPAND, 1);
+	FlexGridSizer1->Add(FlexGridSizer2, 1, wxEXPAND, 0);
 	SetSizer(FlexGridSizer1);
 	MenuBar1 = new wxMenuBar();
 	Menu1 = new wxMenu();
@@ -613,6 +607,10 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	mnuSceneStats = new wxMenuItem(
 		Menu1, ID_MENUITEM18, _("Scene stats"), wxEmptyString, wxITEM_NORMAL);
 	Menu1->Append(mnuSceneStats);
+	mnuPrintScene = new wxMenuItem(
+		Menu1, ID_MENUITEM_PRINT_TEXT, _("Print as text to console"),
+		wxEmptyString, wxITEM_NORMAL);
+	Menu1->Append(mnuPrintScene);
 	Menu1->AppendSeparator();
 	MenuItem1 = new wxMenuItem(
 		Menu1, idMenuQuit, _("Quit\tAlt-F4"), _("Quit the application"),
@@ -690,8 +688,8 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	SetMenuBar(MenuBar1);
 	StatusBar1 = new wxStatusBar(this, ID_STATUSBAR1, 0, _T("ID_STATUSBAR1"));
 	int __wxStatusBarWidths_1[4] = {-10, -10, -4, -5};
-	int __wxStatusBarStyles_1[4] = {wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL,
-									wxSB_NORMAL};
+	int __wxStatusBarStyles_1[4] = {
+		wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL, wxSB_NORMAL};
 	StatusBar1->SetFieldsCount(4, __wxStatusBarWidths_1);
 	StatusBar1->SetStatusStyles(4, __wxStatusBarStyles_1);
 	SetStatusBar(StatusBar1);
@@ -734,6 +732,7 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 	Bind(wxEVT_MENU, &svf::OnMenuItem14Selected, this, ID_MENUITEM12);
 	Bind(wxEVT_MENU, &svf::OnMenuItemHighResRender, this, ID_MENUITEM23);
 	Bind(wxEVT_MENU, &svf::OnmnuSceneStatsSelected, this, ID_MENUITEM18);
+	Bind(wxEVT_MENU, &svf::OnMenuPrintScene, this, ID_MENUITEM_PRINT_TEXT);
 	Bind(wxEVT_MENU, &svf::OnQuit, this, idMenuQuit);
 	Bind(wxEVT_MENU, &svf::OnmnuSelectNoneSelected, this, ID_MENUITEM24);
 	Bind(wxEVT_MENU, &svf::OnmnuSelectByClassSelected, this, ID_MENUITEM26);
@@ -764,8 +763,8 @@ _DSceneViewerFrame::_DSceneViewerFrame(wxWindow* parent, wxWindowID id)
 #if wxCHECK_VERSION(2, 9, 0)
 	m_canvas->SetMinClientSize(wxSize(100, 100));
 #endif
-	FlexGridSizer1->Add(
-		m_canvas, 1, wxALL | wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 0);
+	m_canvas->setMinimumZoom(0.1);
+	FlexGridSizer1->Add(m_canvas, 1, wxEXPAND, 0);
 
 	// Load an empty scene:
 	wxCommandEvent dummEvent;
@@ -818,7 +817,7 @@ void _DSceneViewerFrame::OnNewScene(wxCommandEvent& event)
 	openGLSceneRef->insert(stock_objects::CornerXYZ());
 
 	// Add a clone viewport:
-	COpenGLViewport::Ptr vi = openGLSceneRef->createViewport("clone");
+	Viewport::Ptr vi = openGLSceneRef->createViewport("clone");
 	vi->setViewportPosition(0.05, 0.05, 0.2, 0.2);
 	vi->setCloneView("main");
 	vi->setCloneCamera(true);
@@ -852,25 +851,18 @@ void _DSceneViewerFrame::OnOpenFile(wxCommandEvent& event)
 void _DSceneViewerFrame::loadFromFile(
 	const std::string& fil, bool isInASequence)
 {
+	WX_START_TRY
 	try
 	{
 		// Save the path
 		saveLastUsedDirectoryToCfgFile(fil);
 
-		// static float
-		// old_cam_pX,old_cam_pY,old_cam_pZ,old_cam_d,old_cam_az,old_cam_el;
-		static bool first = true;
-
-		if (first)
-		{
-			first = false;
-			// old_cam_pX=old_cam_pY=old_cam_pZ=old_cam_d=old_cam_az=old_cam_el=0.0f;
-		}
-
 		CFileGZInputStream f(fil);
 
+		const auto oldCanvasCamera = m_canvas->cameraParams();
+
 		static mrpt::system::CTicTac tictac;
-		auto openGLSceneRef = m_canvas->getOpenGLSceneRef();
+		auto& openGLSceneRef = m_canvas->getOpenGLSceneRef();
 		{
 			std::lock_guard<std::mutex> lock(critSec_UpdateScene);
 
@@ -895,11 +887,9 @@ void _DSceneViewerFrame::loadFromFile(
 		// Change the camera if necesary:
 		if (openGLSceneRef->followCamera())
 		{
-			COpenGLViewport::Ptr view = openGLSceneRef->getViewport("main");
-			if (!view)
-				THROW_EXCEPTION(
-					"Fatal error: there is no 'main' viewport in the 3D "
-					"scene!");
+			Viewport::Ptr view = openGLSceneRef->getViewport("main");
+			ASSERTMSG_(
+				view, "ERROR: there is no 'main' viewport in the 3D scene!");
 
 			CCamera::Ptr cam = openGLSceneRef->getByClass<CCamera>();
 
@@ -926,6 +916,10 @@ void _DSceneViewerFrame::loadFromFile(
 			// Remove the camera from the object:
 			if (camIsCCameraObj) openGLSceneRef->removeObject(cam);
 		}
+		else
+		{
+			m_canvas->setCameraParams(oldCanvasCamera);
+		}
 
 		loadedFileName = fil;
 
@@ -942,24 +936,20 @@ void _DSceneViewerFrame::loadFromFile(
 			(format("File loaded in %.03fs", timeToLoad).c_str()), 0);
 
 		Refresh(false);
+		Update();
 	}
-	catch (const std::exception& e)
-	{
-		std::cerr << mrpt::exception_to_str(e) << std::endl;
-		btnAutoplay->SetValue(false);
-		wxMessageBox(mrpt::exception_to_str(e), _("Exception"), wxOK, this);
-	}
-	catch (...)
+	catch (const std::exception&)
 	{
 		btnAutoplay->SetValue(false);
-		wxMessageBox(_("Runtime error!"), _("Exception"), wxOK, this);
+		throw;
 	}
+	WX_END_TRY
 }
 
 void _DSceneViewerFrame::updateTitle()
 {
 	SetTitle((format(
-				  "3DSceneViewer - Part of the MRPT project [%s]",
+				  "SceneViewer3D - Part of the MRPT project [%s]",
 				  (extractFileName(loadedFileName) + string(".") +
 				   extractFileExtension(loadedFileName))
 					  .c_str())
@@ -968,13 +958,14 @@ void _DSceneViewerFrame::updateTitle()
 
 void _DSceneViewerFrame::OntimLoadFileCmdLineTrigger(wxTimerEvent&)
 {
-	timLoadFileCmdLine.Stop();  // One shot only.
+	timLoadFileCmdLine.Stop();	// One shot only.
 	// Open file if passed by the command line:
 	if (!global_fileToOpen.empty())
 	{
 		if (mrpt::system::strCmpI(
-				"3Dscene", mrpt::system::extractFileExtension(
-							   global_fileToOpen, true /*ignore .gz*/)))
+				"3Dscene",
+				mrpt::system::extractFileExtension(
+					global_fileToOpen, true /*ignore .gz*/)))
 		{
 			loadFromFile(global_fileToOpen);
 		}
@@ -1016,17 +1007,19 @@ void _DSceneViewerFrame::OntimAutoplay(wxTimerEvent& event)
 
 	// Continue?
 	if (btnAutoplay->GetValue())
-		m_autoplayTimer->Start(delayBetweenAutoplay, true);  // One-shot:
+		m_autoplayTimer->Start(delayBetweenAutoplay, true);	 // One-shot:
 }
 
 void _DSceneViewerFrame::OnMenuBackColor(wxCommandEvent& event)
 {
 	wxColourData colourData;
 	wxColour color;
-	color.Set(
-		(int)(255 * m_canvas->getClearColorR()),
-		(int)(255 * m_canvas->getClearColorG()),
-		(int)(255 * m_canvas->getClearColorB()));
+
+	auto bkCol = m_canvas->getOpenGLSceneRef()
+					 ->getViewport()
+					 ->getCustomBackgroundColor();
+
+	color.Set((int)(255 * bkCol.R), (int)(255 * bkCol.G), (int)(255 * bkCol.B));
 
 	colourData.SetColour(color);
 	colourData.SetChooseFull(true);
@@ -1036,8 +1029,9 @@ void _DSceneViewerFrame::OnMenuBackColor(wxCommandEvent& event)
 	if (wxID_OK == colDial.ShowModal())
 	{
 		wxColour col = colDial.GetColourData().GetColour();
-		m_canvas->setClearColors(
-			col.Red() / 255.0f, col.Green() / 255.0f, col.Blue() / 255.0f);
+
+		m_canvas->getOpenGLSceneRef()->getViewport()->setCustomBackgroundColor(
+			{col.Red() / 255.0f, col.Green() / 255.0f, col.Blue() / 255.0f});
 
 		int w, h;
 		m_canvas->GetSize(&w, &h);
@@ -1221,7 +1215,7 @@ void _DSceneViewerFrame::OnTravellingTrigger(wxTimerEvent& event)
 		else
 		{
 			// Change the camera
-			COpenGLViewport::Ptr view = openGLSceneRef->getViewport("main");
+			Viewport::Ptr view = openGLSceneRef->getViewport("main");
 
 			if (!view)
 				THROW_EXCEPTION(
@@ -1257,8 +1251,7 @@ void _DSceneViewerFrame::OnTravellingTrigger(wxTimerEvent& event)
 					}
 				}
 
-				if (valid)
-					m_canvas->setCameraPointing(p.x(), p.y(), p.z());
+				if (valid) m_canvas->setCameraPointing(p.x(), p.y(), p.z());
 				else
 				{
 					// end of path:
@@ -1310,7 +1303,7 @@ void _DSceneViewerFrame::OnStartCameraTravelling(wxCommandEvent& event)
 		else
 		{
 			// Change the camera
-			COpenGLViewport::Ptr view = openGLSceneRef->getViewport("main");
+			Viewport::Ptr view = openGLSceneRef->getViewport("main");
 
 			if (!view)
 				THROW_EXCEPTION(
@@ -1353,7 +1346,7 @@ void _DSceneViewerFrame::OnStartCameraTravelling(wxCommandEvent& event)
 
 			maxv = azimuth + max_value;
 
-			m_travelling_is_arbitrary = false;  // Circular
+			m_travelling_is_arbitrary = false;	// Circular
 			m_tTravelling.Start(100);
 
 			Refresh(false);
@@ -1453,12 +1446,13 @@ void _DSceneViewerFrame::OnmnuItemChangeMaxPointsPerOctreeNodeSelected(
 		_("Max. density of points in each octree (points/pixel^2):"),
 		_("Enter new value"),
 		wxString::Format(
-			_("%e"), (double)mrpt::global_settings::
-						 OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL()),
+			_("%e"),
+			(double)mrpt::global_settings::
+				OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL()),
 		this);
 
 	double N1, N2;
-	if (sRet1.ToDouble(&N1) && sRet2.ToDouble(&N2))
+	if (sRet1.ToCDouble(&N1) && sRet2.ToCDouble(&N2))
 	{
 		mrpt::global_settings::OCTREE_RENDER_MAX_POINTS_PER_NODE(N1);
 		mrpt::global_settings::OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL(N2);
@@ -1535,8 +1529,21 @@ void func_gather_stats(const mrpt::opengl::CRenderizable::Ptr& o)
 	}
 }
 
+void _DSceneViewerFrame::OnMenuPrintScene(wxCommandEvent&)
+{
+	try
+	{
+		auto d = m_canvas->getOpenGLSceneRef()->asYAML();
+		d.printAsYAML(std::cout);
+	}
+	catch (const std::exception& e)
+	{
+		wxMessageBox(mrpt::exception_to_str(e), _("Exception"), wxOK, this);
+	}
+}
+
 // Gather stats on the scene:
-void _DSceneViewerFrame::OnmnuSceneStatsSelected(wxCommandEvent& event)
+void _DSceneViewerFrame::OnmnuSceneStatsSelected(wxCommandEvent&)
 {
 	try
 	{
@@ -1568,7 +1575,7 @@ void _DSceneViewerFrame::OnmnuSceneStatsSelected(wxCommandEvent& event)
 }
 
 static const std::string name_octrees_bb_globj =
-	"__3dsceneviewer_gl_octree_bb__";
+	"__SceneViewer3D_gl_octree_bb__";
 CSetOfObjects::Ptr aux_gl_octrees_bb;
 
 void func_get_octbb(const mrpt::opengl::CRenderizable::Ptr& o)
@@ -1698,7 +1705,7 @@ void _DSceneViewerFrame::OnMenuItemImportPLYPointCloud(wxCommandEvent& event)
 		{
 			auto openGLSceneRef = m_canvas->getOpenGLSceneRef();
 			// Set the point cloud as the only object in scene:
-			openGLSceneRef = std::make_shared<opengl::COpenGLScene>();
+			openGLSceneRef = std::make_shared<opengl::Scene>();
 
 			if (dlgPLY.cbXYGrid->GetValue())
 			{
@@ -1713,7 +1720,7 @@ void _DSceneViewerFrame::OnMenuItemImportPLYPointCloud(wxCommandEvent& event)
 					mrpt::opengl::stock_objects::CornerXYZ());
 
 			double ptSize;
-			dlgPLY.cbPointSize->GetStringSelection().ToDouble(&ptSize);
+			dlgPLY.cbPointSize->GetStringSelection().ToCDouble(&ptSize);
 			if (gl_points) gl_points->setPointSize(ptSize);
 			if (gl_points_col) gl_points_col->setPointSize(ptSize);
 
@@ -1721,23 +1728,17 @@ void _DSceneViewerFrame::OnMenuItemImportPLYPointCloud(wxCommandEvent& event)
 			{
 				switch (dlgPLY.rbIntFromXYZ->GetSelection())
 				{
-					case 1:
-						gl_points->enableColorFromX();
-						break;
-					case 2:
-						gl_points->enableColorFromY();
-						break;
-					case 3:
-						gl_points->enableColorFromZ();
-						break;
+					case 1: gl_points->enableColorFromX(); break;
+					case 2: gl_points->enableColorFromY(); break;
+					case 3: gl_points->enableColorFromZ(); break;
 				};
 			}
 
 			TPose3D ptCloudPose(0, 0, 0, 0, 0, 0);
 
-			dlgPLY.edYaw->GetValue().ToDouble(&ptCloudPose.yaw);
-			dlgPLY.edPitch->GetValue().ToDouble(&ptCloudPose.pitch);
-			dlgPLY.edRoll->GetValue().ToDouble(&ptCloudPose.roll);
+			dlgPLY.edYaw->GetValue().ToCDouble(&ptCloudPose.yaw);
+			dlgPLY.edPitch->GetValue().ToCDouble(&ptCloudPose.pitch);
+			dlgPLY.edRoll->GetValue().ToCDouble(&ptCloudPose.roll);
 			ptCloudPose.yaw = DEG2RAD(ptCloudPose.yaw);
 			ptCloudPose.pitch = DEG2RAD(ptCloudPose.pitch);
 			ptCloudPose.roll = DEG2RAD(ptCloudPose.roll);
@@ -1882,15 +1883,16 @@ void _DSceneViewerFrame::OnMenuItemHighResRender(wxCommandEvent& event)
 			std::string sH = std::string(ssH.mb_str());
 			const long height = atoi(sH.c_str());
 
-			CFBORender render(width, height, true /* skip Glut extra window */);
+			CFBORender::Parameters params;
+			params.width = width;
+			params.height = height;
+			params.create_EGL_context = false;	// dont, will reuse existing one
+
+			CFBORender render(params);
 			CImage frame(width, height, CH_RGB);
 
-			render.setBackgroundColor(mrpt::img::TColorf(
-				m_canvas->getClearColorR(), m_canvas->getClearColorG(),
-				m_canvas->getClearColorB(), 1.0));
-
 			// render the scene
-			render.getFrame(*m_canvas->getOpenGLSceneRef(), frame);
+			render.render_RGB(*m_canvas->getOpenGLSceneRef(), frame);
 
 			frame.saveToFile(sTargetFil);
 		}
@@ -2079,32 +2081,32 @@ void _DSceneViewerFrame::OnmnuImportLASSelected(wxCommandEvent& event)
 			return;
 		}
 
-		mrpt::math::TPoint3D bb_min, bb_max;
+		mrpt::math::TBoundingBox bb;
 		{
 			wxBusyCursor busy;
 			if (gl_points_col)
 			{
 				gl_points_col->loadFromPointsMap(&pts_map);
-				gl_points_col->getBoundingBox(bb_min, bb_max);
+				bb = gl_points_col->getBoundingBox();
 			}
 			else
 			{
 				gl_points->loadFromPointsMap(&pts_map);
-				gl_points->getBoundingBox(bb_min, bb_max);
+				bb = gl_points->getBoundingBox();
 			}
 		}
 
-		const double scene_size = bb_min.distanceTo(bb_max);
+		const double scene_size = bb.min.distanceTo(bb.max);
 
 		// Set the point cloud as the only object in scene:
-		auto scene = std::make_shared<opengl::COpenGLScene>();
+		auto scene = std::make_shared<opengl::Scene>();
 		m_canvas->setOpenGLSceneRef(scene);
 
 		if (dlgPLY.cbXYGrid->GetValue())
 		{
 			mrpt::opengl::CGridPlaneXY::Ptr obj =
 				mrpt::opengl::CGridPlaneXY::Create(
-					bb_min.x, bb_max.x, bb_min.y, bb_max.y, 0,
+					bb.min.x, bb.max.x, bb.min.y, bb.max.y, 0,
 					scene_size * 0.02);
 			obj->setColor(0.3f, 0.3f, 0.3f);
 			scene->insert(obj);
@@ -2114,7 +2116,7 @@ void _DSceneViewerFrame::OnmnuImportLASSelected(wxCommandEvent& event)
 			scene->insert(mrpt::opengl::stock_objects::CornerXYZ());
 
 		double ptSize;
-		dlgPLY.cbPointSize->GetStringSelection().ToDouble(&ptSize);
+		dlgPLY.cbPointSize->GetStringSelection().ToCDouble(&ptSize);
 		if (gl_points) gl_points->setPointSize(ptSize);
 		if (gl_points_col) gl_points_col->setPointSize(ptSize);
 
@@ -2122,23 +2124,17 @@ void _DSceneViewerFrame::OnmnuImportLASSelected(wxCommandEvent& event)
 		{
 			switch (dlgPLY.rbIntFromXYZ->GetSelection())
 			{
-				case 1:
-					gl_points->enableColorFromX();
-					break;
-				case 2:
-					gl_points->enableColorFromY();
-					break;
-				case 3:
-					gl_points->enableColorFromZ();
-					break;
+				case 1: gl_points->enableColorFromX(); break;
+				case 2: gl_points->enableColorFromY(); break;
+				case 3: gl_points->enableColorFromZ(); break;
 			};
 		}
 
 		TPose3D ptCloudPose(0, 0, 0, 0, 0, 0);
 
-		dlgPLY.edYaw->GetValue().ToDouble(&ptCloudPose.yaw);
-		dlgPLY.edPitch->GetValue().ToDouble(&ptCloudPose.pitch);
-		dlgPLY.edRoll->GetValue().ToDouble(&ptCloudPose.roll);
+		dlgPLY.edYaw->GetValue().ToCDouble(&ptCloudPose.yaw);
+		dlgPLY.edPitch->GetValue().ToCDouble(&ptCloudPose.pitch);
+		dlgPLY.edRoll->GetValue().ToCDouble(&ptCloudPose.roll);
 		ptCloudPose.yaw = DEG2RAD(ptCloudPose.yaw);
 		ptCloudPose.pitch = DEG2RAD(ptCloudPose.pitch);
 		ptCloudPose.roll = DEG2RAD(ptCloudPose.roll);
@@ -2151,14 +2147,14 @@ void _DSceneViewerFrame::OnmnuImportLASSelected(wxCommandEvent& event)
 		if (gl_points_col) scene->insert(gl_points_col);
 
 		m_canvas->setCameraPointing(
-			(bb_min.x + bb_max.x) * 0.5, (bb_min.y + bb_max.y) * 0.5,
-			(bb_min.z + bb_max.z) * 0.5);
+			(bb.min.x + bb.max.x) * 0.5, (bb.min.y + bb.max.y) * 0.5,
+			(bb.min.z + bb.max.z) * 0.5);
 
 		m_canvas->setZoomDistance(2 * scene_size);
 		m_canvas->setAzimuthDegrees(45);
 		m_canvas->setElevationDegrees(30);
 
-		COpenGLViewport::Ptr gl_view = scene->getViewport();
+		Viewport::Ptr gl_view = scene->getViewport();
 		gl_view->setViewportClipDistances(0.01, 10 * scene_size);
 
 		loadedFileName =
@@ -2174,9 +2170,9 @@ void _DSceneViewerFrame::OnmnuImportLASSelected(wxCommandEvent& event)
 		std::stringstream ss;
 		ss << pts_map.size() << " points loaded.\n"
 		   << "Bounding box:\n"
-		   << " X: " << bb_min.x << " <=> " << bb_max.x << "\n"
-		   << " Y: " << bb_min.y << " <=> " << bb_max.y << "\n"
-		   << " Z: " << bb_min.z << " <=> " << bb_max.z << "\n"
+		   << " X: " << bb.min.x << " <=> " << bb.max.x << "\n"
+		   << " Y: " << bb.min.y << " <=> " << bb.max.y << "\n"
+		   << " Z: " << bb.min.z << " <=> " << bb.max.z << "\n"
 		   << "LAS header info:\n"
 		   << "---------------------------------\n"
 		   << "FileSignature      : " << las_hdr.FileSignature << endl
@@ -2202,7 +2198,7 @@ void _DSceneViewerFrame::OnmnuImportImageView(wxCommandEvent&)
 {
 	try
 	{
-		auto scene = std::make_shared<opengl::COpenGLScene>();
+		auto scene = std::make_shared<opengl::Scene>();
 		m_canvas->setOpenGLSceneRef(scene);
 		wxFileDialog dialog(
 			this, _("Choose the LAS file to import"),
@@ -2221,7 +2217,7 @@ void _DSceneViewerFrame::OnmnuImportImageView(wxCommandEvent&)
 		if (!im.loadFromFile(fil))
 			THROW_EXCEPTION_FMT("Error loading image file: '%s'", fil.c_str());
 
-		COpenGLViewport::Ptr gl_view = scene->getViewport();
+		Viewport::Ptr gl_view = scene->getViewport();
 		gl_view->setImageView(im);
 
 		Refresh();
