@@ -2,13 +2,13 @@
    |                     Mobile Robot Programming Toolkit (MRPT)            |
    |                          https://www.mrpt.org/                         |
    |                                                                        |
-   | Copyright (c) 2005-2020, Individual contributors, see AUTHORS file     |
+   | Copyright (c) 2005-2023, Individual contributors, see AUTHORS file     |
    | See: https://www.mrpt.org/Authors - All rights reserved.               |
    | Released under BSD License. See: https://www.mrpt.org/License          |
    +------------------------------------------------------------------------+ */
 
 #include "hmtslam-precomp.h"  // Precomp header
-
+//
 #include <mrpt/core/aligned_allocator.h>
 #include <mrpt/graphslam/levmarq.h>
 #include <mrpt/hmtslam/CRobotPosesGraph.h>
@@ -116,8 +116,7 @@ const CHMHMapNode::Ptr CHierarchicalMapMHPartition::getNodeByLabel(
   ---------------------------------------------------------------*/
 CHMHMapNode::Ptr CHierarchicalMapMHPartition::getFirstNode()
 {
-	if (m_nodes.empty())
-		return CHMHMapNode::Ptr();
+	if (m_nodes.empty()) return CHMHMapNode::Ptr();
 	else
 		return (m_nodes.begin())->second;
 }
@@ -748,7 +747,7 @@ void CHierarchicalMapMHPartition::findPathBetweenNodes(
 	14                        previous[v] := u
 	*/
 
-	map<CHMHMapNode::TNodeID, TDistance> d;  // distance
+	map<CHMHMapNode::TNodeID, TDistance> d;	 // distance
 	map<CHMHMapNode::TNodeID, TPrevious> previous;
 	map<CHMHMapNode::TNodeID, CHMHMapArc::Ptr> previous_arcs;
 	map<CHMHMapNode::TNodeID, bool> visited;
@@ -817,8 +816,7 @@ void CHierarchicalMapMHPartition::findPathBetweenNodes(
 			}
 			else
 			{
-				if ((*i)->getNodeFrom() != nodeU->getID())
-					continue;
+				if ((*i)->getNodeFrom() != nodeU->getID()) continue;
 				else
 					vID = (*i)->getNodeTo();
 			}
@@ -918,7 +916,7 @@ void CHierarchicalMapMHPartition::computeCoordinatesTransformationBetweenNodes(
 			(*arcsIt)->m_annotations.get(ARC_ANNOTATION_DELTA, hypothesisID);
 		ASSERT_(anotation);
 
-		CPose3DPDFGaussian pdf;  // Convert to gaussian
+		CPose3DPDFGaussian pdf;	 // Convert to gaussian
 		pdf.copyFrom(*std::dynamic_pointer_cast<CPose3DPDF>(anotation));
 
 		vector<CVectorDouble> samples;
@@ -960,8 +958,7 @@ void CHierarchicalMapMHPartition::computeCoordinatesTransformationBetweenNodes(
 
 		for (unsigned int j = 0; j < pathLength; j++)
 		{
-			if (j)
-				sample = sample + listSamples[j][i];
+			if (j) sample = sample + listSamples[j][i];
 			else
 				sample = listSamples[j][i];
 		}
@@ -1074,8 +1071,7 @@ bool CHierarchicalMapMHPartition::areNodesNeightbour(
 			if ((*itArc)->m_nodeFrom == node2id ||
 				(*itArc)->m_nodeTo == node2id)
 			{
-				if (!requiredAnnotation)
-					return true;
+				if (!requiredAnnotation) return true;
 				else if ((*itArc)->m_annotations.get(
 							 requiredAnnotation, hypothesisID))
 					return true;
@@ -1092,7 +1088,7 @@ bool CHierarchicalMapMHPartition::areNodesNeightbour(
   ---------------------------------------------------------------*/
 
 void CHierarchicalMapMHPartition::getAs3DScene(
-	COpenGLScene& outScene, const CHMHMapNode::TNodeID& idReferenceNode,
+	Scene& outScene, const CHMHMapNode::TNodeID& idReferenceNode,
 	const THypothesisID& hypothesisID,
 	const unsigned int& numberOfIterationsForOptimalGlobalPoses,
 	bool showRobotPoseIDs) const
@@ -1166,11 +1162,11 @@ void CHierarchicalMapMHPartition::getAs3DScene(
 		CMultiMetricMap::Ptr metricMap =
 			node->m_annotations.getAs<CMultiMetricMap>(
 				NODE_ANNOTATION_METRIC_MAPS, hypothesisID);
-		if (metricMap)  // ASSERT_(metricMap);
+		if (metricMap)	// ASSERT_(metricMap);
 		{
 			mrpt::opengl::CSetOfObjects::Ptr objTex =
 				mrpt::opengl::CSetOfObjects::Create();
-			metricMap->getAs3DObject(objTex);
+			metricMap->getVisualizationInto(*objTex);
 			objTex->setPose(pose);
 			outScene.insert(objTex);
 		}
@@ -1302,7 +1298,7 @@ void CHierarchicalMapMHPartition::computeGloballyConsistentNodeCoordinates(
 
 	nodePoses.clear();
 
-	if (m_arcs.empty()) return;  // Nothing we can do!
+	if (m_arcs.empty()) return;	 // Nothing we can do!
 
 	// 1) Convert hmt-slam graph into graphslam graph... (this should be avoided
 	// in future version of HTML-SLAM!!)
@@ -1349,7 +1345,7 @@ void CHierarchicalMapMHPartition::computeGloballyConsistentNodeCoordinates(
 		// To the output map:
 		CPose3DPDFGaussian& new_pose = nodePoses[node_id];
 		new_pose.mean = it_node->second;
-		new_pose.cov.setIdentity();  // *** At present, graphslam does not
+		new_pose.cov.setIdentity();	 // *** At present, graphslam does not
 		// output the uncertainty of poses... ***
 	}
 
